package com.muyu.datasources.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.utils.bean.BeanUtils;
import com.muyu.common.domain.AssetAccredit;
import com.muyu.common.domain.req.AssetAccreditReq;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.common.system.domain.SysDept;
import com.muyu.datasources.service.AssetAccreditService;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author：zhangzhihao
 * @name：assetAccreditController
 * @Date：2024/8/30 9:37
 * 不准抄代码，添加注释，清楚每一行代码意思
 */
@RestController
@RequestMapping("assetAccredit")
public class AssetAccreditController {


    @Autowired
    private AssetAccreditService assetAccreditService;



    @PostMapping("/findUserIdList")
    public Result findUserIdList(@RequestBody AssetAccreditReq req){

        List<Long> list = assetAccreditService.findUserIdList(req);
        return Result.success(list);
    }



    /**
     * 添加用户权限方法
     * @param req
     * @return
     */
    @Log(title = "添加用户权限",businessType = BusinessType.INSERT)
    @PostMapping("/addUserAssetAccredit")
    public Result addUserAssetAccredit(@RequestBody AssetAccreditReq req){
        AssetAccredit assetAccredit = new AssetAccredit();
        assetAccredit.setUserId(req.getUserId());
        assetAccredit.setTableId(req.getTableId());
        assetAccredit.setBasicId(req.getBasicId());

        boolean save = assetAccreditService.save(assetAccredit);
        return save?Result.success():Result.error();
    }

    /**
     * 删除用户权限方法
     * @param req
     * @return
     */
    @Log(title = "删除用户权限",businessType = BusinessType.DELETE)
    @PostMapping("/delUserAssetAccredit")
    public Result delUserAssetAccredit(@RequestBody AssetAccreditReq req){
        int i = assetAccreditService.delUserAssetAccredit(req);

        return i>0?Result.success():Result.error();
    };


    @PostMapping("/findDeptIdList")
    public Result findDeptIdList(@RequestBody AssetAccreditReq req){

        List<Long> list = assetAccreditService.findDeptIdList(req);
        return Result.success(list);
    }





    /**
     * 添加部门权限方法
     * @param req
     * @return
     */
    @Log(title = "添加部门权限",businessType = BusinessType.INSERT)
    @PostMapping("/addDeptAssetAccredit")
    public Result addDeptAssetAccredit(@RequestBody AssetAccreditReq req){
        AssetAccredit assetAccredit = new AssetAccredit();
        assetAccredit.setDeptId(req.getDeptId());
        assetAccredit.setTableId(req.getTableId());
        assetAccredit.setBasicId(req.getBasicId());

        boolean save = assetAccreditService.save(assetAccredit);
        return save?Result.success():Result.error();
    }

    /**
     * 删除部门权限方法
     * @param req
     * @return
     */
    @Log(title = "删除部门权限",businessType = BusinessType.DELETE)
    @PostMapping("/delDeptAssetAccredit")
    public Result delDeptAssetAccredit(@RequestBody AssetAccreditReq req){
        int i = assetAccreditService.delDeptAssetAccredit(req);

        return i>0?Result.success():Result.error();
    };



}
