package com.muyu.datasources.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.domain.DataSources;
import com.muyu.common.domain.req.DataSourcesReq;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.datasources.service.DataSourcesService;

import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.sql.*;
import java.util.List;

/**
 * @Author：zhangzhihao
 * @name：DataSourcesController
 * @Date：2024/8/21 14:47
 * 不准抄代码，添加注释，清楚每一行代码意思
 */
@RestController
@RequestMapping("datasources")
public class DataSourcesController {

    @Autowired
    private DataSourcesService dataSourcesService;

    /**
     * 通过ID查询单条数据
     * @param  id 主键
     * @return 实例对象
     */
    @GetMapping("/{id}")
    public Result<DataSources> queryById(@PathVariable("id") Long id){
        return Result.success(dataSourcesService.getById(id));
    }

    /**
     * 查询所有已有数据源
     * @param req
     * @return
     */
    @PostMapping("/all")
    public Result<Page<DataSources>> all(@RequestBody DataSourcesReq req){
        Page<DataSources> dataSourcesPage = dataSourcesService.selectList(req);
        return Result.success(dataSourcesPage);
    }


    /**
     * 新增数据来源
     * @param dataSources 实例对象
     * @return 实例对象
     */
    @Log(title = "新增数据来源",businessType = BusinessType.INSERT)
    @PostMapping("/addDataSources")
    public Result<DataSources> add(@RequestBody DataSources dataSources){
        dataSourcesService.save(dataSources);
        return Result.success();
    }

    /**
     * 更新数据
     *
     * @param dataSources 实例对象
     * @return 实例对象
     */
    @Log(title = "更新数据来源配置",businessType = BusinessType.UPDATE)
    @PutMapping("/updateDataSources")
    public Result<DataSources> edit(@RequestBody DataSources dataSources){
        dataSourcesService.updateById(dataSources);
        return Result.success();
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @Log(title = "删除数据来源",businessType = BusinessType.DELETE)
    @DeleteMapping("/{id}")
    public Result<Boolean> deleteById(@PathVariable("id") Long id){
        return Result.success(dataSourcesService.removeById(id));
    }


    /**
     * 测试连接
     * @param dataSources
     * @return
     */
    @PostMapping("/connectDataSources")
    public Result connectDataSources(@RequestBody DataSources dataSources){
        int i= dataSourcesService.connectDataSources(dataSources);
        return i>0?Result.success():Result.error();
    }


    /**
     * 同步数据结构
     * @param dataSources
     * @return
     */
    @Log(title = "同步数据结构",businessType = BusinessType.INSERT)
    @PostMapping("/syncAssetStructure")
    public Result syncAssetStructure(@RequestBody DataSources dataSources){
        Integer i = dataSourcesService.syncAssetStructure(dataSources);
        return Result.success(i);
    }

    /**
     * 导出数据
     * @param response
     */
    @PostMapping("/export")
    public void export(HttpServletResponse response){
        List<DataSources> list = dataSourcesService.list();

        ExcelUtil<DataSources> util = new ExcelUtil<>(DataSources.class);
        util.exportExcel(response,list,"连接数据库配置");
    }

//    @PostMapping("/test")
//    public Result  test() {
//        // Oracle数据库的JDBC URL
//        String jdbcUrl = "jdbc:oracle:thin:@LAPTOP-A6NCJEU3:1521:XE";
//
//
//        String username = "zzh";
//        String password = "123456";
//
//        Connection conn = null;
//        Statement stmt = null;
//        ResultSet rs = null;
//        String str="";
//        try {
//            // 加载Oracle JDBC驱动
//            Class.forName("oracle.jdbc.driver.OracleDriver");
//
//            // 建立数据库连接
//            conn = DriverManager.getConnection(jdbcUrl, username, password);
//
//            // 创建Statement对象
//            stmt = conn.createStatement();
//
//            // 编写SQL查询语句，从ZZH模式的T_USER表中查询NAME字段
//            String sql = "SELECT TABLE_NAME FROM DBA_TABLES WHERE OWNER = 'ZZH'";
//
//            // 执行查询
//            rs = stmt.executeQuery(sql);
//
//
//            // 遍历ResultSet对象，获取NAME字段的值
//            while (rs.next()) {
//                String name = rs.getString(1);
//                str+=name;
//                System.out.println("NAME: " + name);
//            }
//
//        }catch (ClassNotFoundException e) {
//            System.out.println("Oracle JDBC Driver类未找到！");
//            e.printStackTrace();
//        } catch (SQLException e) {
//            System.out.println("数据库连接失败或SQL查询出错！");
//            e.printStackTrace();
//        }  finally {
//            // 关闭资源
//            try {
//                if (rs != null) {
//                    rs.close();
//                }
//                if (stmt != null) {
//                    stmt.close();
//                }
//                if (conn != null) {
//                    conn.close();
//                }
//            } catch (SQLException e) {
//                e.printStackTrace();
//            }
//        }
//
//        return Result.success(str);
//    }

}
