package com.muyu.datasources.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.client.mysql.MySqlDataSource;
import com.muyu.client.mysql.MySqlQuery;
import com.muyu.common.data.base.BaseQueryHandler;
import com.muyu.common.domain.DataSources;
import com.muyu.common.domain.TableInfo;

import com.muyu.common.pool.MysqlPool;
import com.muyu.datasources.mapper.DataValueMapper;

import com.muyu.datasources.service.DataValueService;
import com.muyu.datasources.service.DataSourcesService;
import com.muyu.datasources.service.TableInfoService;
import com.muyu.etl.domain.DataValue;
import com.muyu.etl.enums.DataType;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import static com.muyu.common.pool.base.BaseConfig.SELECTALL;

/**
 * @Author：zhangzhihao
 * @name：DataKltvServiceImpl
 * @Date：2024/8/27 15:37
 * 不准抄代码，添加注释，清楚每一行代码意思
 */
@Log4j2
@Service
public class DataValueServiceImpl extends ServiceImpl<DataValueMapper, DataValue> implements DataValueService {

    @Autowired
    private DataSourcesService dataSourcesService;


    @Autowired
    private TableInfoService tableInfoService;





@Override
/**
 * 根据给定的数据库ID和SQL语句查询表数据
 *
 * @param basicId 数据库ID，用于获取对应的数据库连接
 * @param sql     执行的SQL语句
 * @return 返回查询结果，包含每个字段的数据值列表
 */
public List<List<DataValue>> findTableValue(Long basicId, String sql) {

    // 根据数据库ID获取数据库配置信息
    DataSources dataSources = dataSourcesService.getById(basicId);
    // 创建MySQL连接池并初始化
    MysqlPool mysqlPool = new MysqlPool(dataSources);
    mysqlPool.init();
    // 获取数据库连接
    Connection conn = mysqlPool.getConn();

    // 创建列表用于存储查询结果
    List<List<DataValue>> list = new ArrayList<>();

    // 使用try-with-resources确保连接最终被关闭
    try (conn; PreparedStatement preparedStatement = conn.prepareStatement(sql)) {
        // 执行查询并获取结果集
        ResultSet resultSet = preparedStatement.executeQuery();
        // 获取结果集的元数据
        ResultSetMetaData metaData = resultSet.getMetaData();
        // 获取列数
        int columnCount = metaData.getColumnCount();

        // 使用ConcurrentHashMap存储列索引与DataValue对象的映射
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        // 遍历结果集中的每一行
        while (resultSet.next()) {
            // 创建列表存储当前行的数据值
            ArrayList<DataValue> dataValues = new ArrayList<>();
            // 遍历每一列
            for (int i = 1; i <= columnCount; i++) {
                // 如果是结果集的第一行，初始化DataValue对象
                if (resultSet.isFirst()) {
                    // 获取列的类型名称
                    String columnTypeName = metaData.getColumnTypeName(i);
                    // 获取数据库元数据
                    DatabaseMetaData metaDataColumns = conn.getMetaData();
                    // 获取列的详细信息
                    ResultSet columns = metaDataColumns.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                    // 初始化备注信息
                    String remarks = null;

                    // 从列的详细信息中获取备注
                    while (columns.next()) {
                        remarks = columns.getString("REMARKS");
                        log.info("字段备注: " + remarks);
                    }

                    // 构建DataValue对象并添加到列表中
                    DataValue build = DataValue.builder()
                            .key(metaData.getColumnName(i))
                            .label(remarks)
                            .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                            .type(DataType.findBySqlType(columnTypeName))
                            .build();
                    dataValues.add(build);
                    map.put(i, build);
                } else {
                    // 非第一行时，使用已有的DataValue对象更新数据值
                    DataValue build = DataValue.builder()
                            .key(metaData.getColumnName(i))
                            .label(map.get(i).getLabel())
                            .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                            .type(map.get(i).getType())
                            .build();
                    dataValues.add(build);
                }
            }
            // 将当前行的数据值列表添加到结果列表中
            list.add(dataValues);
        }
    } catch (SQLException e) {
        // 记录数据库操作日志并抛出运行时异常
        log.error("数据库操作失败", e);
        throw new RuntimeException("数据库操作失败", e);
    } finally {
        // 释放数据库连接并关闭连接池
        mysqlPool.replease(conn);
        mysqlPool.closeConn();
    }

    // 返回查询结果列表
    return list;
}



    @Override
    public List<DataValue> findTableValueByTableName(Long basicId, String tableName) {

        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        DataSources dataSources = dataSourcesService.getById(basicId);
        MysqlPool mysqlPool = new MysqlPool(dataSources);
        mysqlPool.init();
        Connection conn = mysqlPool.getConn();

        List<DataValue> list = new ArrayList<>();

        try {

            PreparedStatement preparedStatement = conn.prepareStatement(SELECTALL+tableName+" LIMIT 10");

            ResultSet resultSet = preparedStatement.executeQuery();
            ResultSetMetaData metaData = resultSet.getMetaData();
            int columnCount = metaData.getColumnCount();


            while (resultSet.next()){
                for (int i = 1; i <= columnCount; i++) {
                    if (resultSet.isFirst()){
                        String columnTypeName = metaData.getColumnTypeName(i);

                        DatabaseMetaData metaDataColumns = conn.getMetaData();
                        ResultSet columns = metaDataColumns.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                        String remarks =null;

                        while (columns.next()){
                            remarks = columns.getString("REMARKS");

                            log.info("字段备注:"+remarks);
                        }
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                                .type(DataType.findBySqlType(columnTypeName))
                                .build();
                        map.put(i,build);
                        list.add(build);
                    }else {
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(map.get(i).getLabel())
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                                .type(map.get(i).getType())
                                .build();
                        list.add(build);
                    }

                }
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        mysqlPool.replease(conn);
        mysqlPool.closeConn();
        return list;
    }

    @Override
    public List<List<DataValue>> findTableValueContributionRule(Long basicId, Long tableId) {
        ConcurrentHashMap<Integer, DataValue> map = new ConcurrentHashMap<>();
        TableInfo dataSourceTableInfo = tableInfoService.getById(basicId);

        DataSources dataSources = dataSourcesService.getById(dataSourceTableInfo.getBasicId());
        TableInfo tableInfo = tableInfoService.getById(tableId);
        String tableName = tableInfo.getTableName();

        MysqlPool mysqlPool = new MysqlPool(dataSources);
        mysqlPool.init();
        Connection conn = mysqlPool.getConn();

        List<List<DataValue>> list = new ArrayList<>();

        try {

            PreparedStatement preparedStatement = conn.prepareStatement(SELECTALL+tableName);

            ResultSet resultSet = preparedStatement.executeQuery();
            ResultSetMetaData metaData = resultSet.getMetaData();
            int columnCount = metaData.getColumnCount();


            while (resultSet.next()){
                ArrayList<DataValue> dataValues = new ArrayList<>();
                for (int i = 1; i <= columnCount; i++) {

                    if (resultSet.isFirst()){
                        String columnTypeName = metaData.getColumnTypeName(i);

                        DatabaseMetaData metaDataColumns = conn.getMetaData();
                        ResultSet columns = metaDataColumns.getColumns(null, null, metaData.getTableName(i), metaData.getColumnName(i));
                        String remarks =null;

                        while (columns.next()){
                            remarks = columns.getString("REMARKS");

                            log.info("字段备注:"+remarks);
                        }
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(remarks)
                                .value(resultSet.getObject(i, DataType.convertType(columnTypeName)))
                                .type(DataType.findBySqlType(columnTypeName))
                                .build();
                        dataValues.add(build);
                        map.put(i,build);
                    }else {
                        DataValue build = DataValue.builder()
                                .key(metaData.getColumnName(i))
                                .label(map.get(i).getLabel())
                                .value(resultSet.getObject(i, map.get(i).getType().getTargetType()))
                                .type(map.get(i).getType())
                                .build();
                        dataValues.add(build);
                    }

                }
                list.add(dataValues);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        mysqlPool.replease(conn);
        mysqlPool.closeConn();
        return list;
    }

    @Override
    public Long findCount(Long basicId, String sql) {


        DataSources dataSources = dataSourcesService.getById(basicId);
        MysqlPool mysqlPool = new MysqlPool(dataSources);
        mysqlPool.init();
        Connection conn = mysqlPool.getConn();
        Long count=0L;
        try {
            PreparedStatement preparedStatement = conn.prepareStatement(sql);
            ResultSet resultSet = preparedStatement.executeQuery();
            while (resultSet.next()){
                 count = resultSet.getLong(1);
            }

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }finally {
            try {
                conn.close();
                mysqlPool.closeConn();
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        }


        return count;
    }

    @Override
    public DataValue[][] findTableValueToArray(Long basicId, String sql, Long one, Integer two) {

        MySqlQuery mySqlQuery = new MySqlQuery();
        mySqlQuery.setSql(sql);
        mySqlQuery.setDataSourceId(basicId);
        mySqlQuery.setOne(one);
        mySqlQuery.setTwo(two);
        BaseQueryHandler.set(mySqlQuery);
        MySqlDataSource mySqlDataSource = new MySqlDataSource();
        DataValue[][] rows = mySqlDataSource.getRows();

        return rows;
    }



}
